package artwork

import (
	"encoding/json"
	"github.com/fonchain_enterprise/fonchain-main/api/artist"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

func RateList(c *gin.Context) {
	var req artwork.RateListReq
	if err := c.ShouldBind(&req); err != nil {
		zap.L().Error("RepairList ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(c, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}

	if len(req.ArtistNum) >= 2 {
		artistData, errS := service.GrpcArtistImpl.ArtistIdName(c, &artist.ArtIdNameRequest{Keyword: req.ArtistNum})
		if errS != nil {
			service.ResponseQuickMsg(c, e.Failed, errS.Error(), nil)
			return
		}
		for _, v := range artistData.Items {
			req.ArtistUuids = append(req.ArtistUuids, v.Uuid)
		}
	}
	listResp, _err := service.GrpcArtworkImpl.RateList(c, &req)
	if _err != nil {
		service.ResponseQuickMsg(c, e.Failed, _err.Error(), nil)
		return
	}
	if len(listResp.Data) == 0 {
		service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), listResp)
		return
	}
	var artistUuids []string
	for _, v := range listResp.Data {
		artistUuids = append(artistUuids, v.ArtistUuid)
	}
	if len(artistUuids) > 0 {
		artistResp, err := service.GrpcArtistImpl.ArtistData(c, &artist.ArtistDataRequest{Uids: artistUuids})
		if err != nil {
			service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
			return
		}
		for _, v := range listResp.Data {
			for _, vv := range artistResp.Data {
				if v.ArtistUuid == vv.ProfileInfo.Uid {
					v.ArtistNum = vv.ProfileInfo.Tnum
				}
			}
		}
	}
	service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), listResp)
	return
}

func UpdateRateStatus(c *gin.Context) {
	var (
		req artwork.UpdateRateStatusReq
		err error
	)
	if err = c.ShouldBind(&req); err != nil {
		zap.L().Error("RepairList ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(c, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if mLoginInfoAny, exists := c.Get("mLoginInfo"); exists {
		info := mLoginInfoAny.(model.LoginInfo)
		info.Extend = nil
		info.PositionUsers = nil
		userInfoByte, _ := json.Marshal(info)
		req.OperatorUserInfo = string(userInfoByte)
	}

	if _, err = service.GrpcArtworkImpl.UpdateRateStatus(c, &req); err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), nil)
	return
}

func ScaleInfo(c *gin.Context) {
	var req artwork.ScaleArtworkInfoReq
	var err error
	if err = c.ShouldBind(&req); err != nil {
		zap.L().Error("RepairList ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(c, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if err = service.CheckArtworkApproval(req.ArtworkUuid); err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	listResp, _err := service.GrpcArtworkImpl.ScaleArtworkInfo(c, &req)
	if _err != nil {
		service.ResponseQuickMsg(c, e.Failed, _err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), listResp)
	return
}
