package service_copyright

import (
	"context"
	"fmt"
	"sync"
	"time"

	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/artist"
	"github.com/fonchain_enterprise/fonchain-main/api/dci"
	"github.com/fonchain_enterprise/fonchain-main/api/digital_copyright"
	"github.com/fonchain_enterprise/fonchain-main/pkg/cache"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	loigc_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/logic/copyright"
	model_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/model/copyright"
	pkgRequest "github.com/fonchain_enterprise/fonchain-main/pkg/request"
	pkgResponse "github.com/fonchain_enterprise/fonchain-main/pkg/response"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

var wg sync.WaitGroup

// 00084a8a-6384-49a6-b88e-fb11935147b5 李益兵
// 8d30e9aa-229d-4906-a93e-8872ad0df10e 许泰学
// 提交申领
func CreateArtworkDci(ctx *gin.Context) {
	var (
		err       error
		createReq *digital_copyright.CreateDciPreregistrationRequest
	)
	if err = ctx.ShouldBind(&createReq); err != nil {
		logger.Errorf("CreateArtworkCopyright err ShouldBind %s", err.Error())
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	var logicCopyright = new(loigc_copyright.Copyright)
	if err = logicCopyright.CreatePreregistration(ctx, createReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	// 定义一个msg结构体 返回数据
	type MsgInfo struct {
		Msg string
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), nil)
	return
}

// 补充信息
func AwAdditionalInfo(ctx *gin.Context) {
	var (
		err  error
		req  pkgRequest.AdditionalRequest
		resp pkgResponse.AdditionalResp
	)
	if err = ctx.ShouldBind(&req); err != nil {
		logger.Errorf("AwAdditionalInfo err ShouldBind %s", err.Error())
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	var logicCopyright = new(loigc_copyright.Copyright)
	if resp, err = logicCopyright.GetAdditionalInfo(req.ArtworkUuid); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp)
	return
}

// 查询申领信息
func DciContentInfo(ctx *gin.Context) {
	var (
		req  pkgRequest.CopyrightIdAll
		resp *dci.QueryDciPreregistrationResponse
		err  error
	)
	if err = ctx.ShouldBind(&req); err != nil {
		logger.Errorf("DciContentInfo err ShouldBind %s", err.Error())
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	var logicCopyright = new(loigc_copyright.Copyright)
	if resp, err = logicCopyright.PreregistrationInfo(req.DciContentId, req.CopyrightId); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if resp.ResultCode != "OK" {
		service.ResponseQuickMsg(ctx, e.Failed, model_copyright.DciUserResultMsg[resp.ResultMsg], resp)
		return
	}

	service.ResponseQuickMsg(ctx, e.Ok, "", resp)
	return
}

func CopyrightList(ctx *gin.Context) {
	var (
		listReq  *digital_copyright.CopyrightListReq
		listResp *digital_copyright.CopyrightListResp
		err      error
	)
	if err = ctx.ShouldBind(&listReq); err != nil {
		zap.L().Error("CopyrightList err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if listResp, err = service.GrpcCopyrightImpl.CopyrightList(context.Background(), listReq); err != nil {
		zap.L().Error("CopyrightList err CopyrightList", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	for _, v := range listResp.Data {
		if v.BlackListStatus == 2 {
			v.ArtistName = fmt.Sprintf("#%s", v.ArtistName)
		}
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), listResp)
	return
}

func CopyrightDetail(ctx *gin.Context) {
	var (
		detailReq  *digital_copyright.CopyrightDetailReq
		detailResp *digital_copyright.CopyrightDetailResp
		err        error
	)
	if err = ctx.ShouldBind(&detailReq); err != nil {
		zap.L().Error("CopyrightDetail err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if detailResp, err = service.GrpcCopyrightImpl.CopyrightDetail(context.Background(), detailReq); err != nil {
		zap.L().Error("CopyrightList err CopyrightDetail", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if detailResp == nil {
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.NOTDATA), nil)
		return
	}
	//如果是待提交的话需要查询补充信息
	if detailResp.BaseInfo != nil && detailResp.AdditionalInfo != nil && (detailResp.AdditionalInfo.CreationCompletionAddress == "" || detailResp.AdditionalInfo.FirstPublicationAddressCode == "") {
		var logicCopyright = new(loigc_copyright.Copyright)
		var additionalInfo pkgResponse.AdditionalResp
		if additionalInfo, err = logicCopyright.GetAdditionalInfo(detailResp.BaseInfo.ArtworkUuid); err != nil {
			zap.L().Error("CreateArtworkCopyrightSave err GetAdditionalInfo", zap.Error(err))
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
			return
		}
		detailResp.AdditionalInfo.CreationCompletionAddressCode = detailResp.WorkInfo.CreateAddressCode
		detailResp.AdditionalInfo.CreationCompletionAddress = detailResp.WorkInfo.CreateAddress
		detailResp.AdditionalInfo.CreationCompletionDate = additionalInfo.CreationCompletionDate
		detailResp.AdditionalInfo.FirstPublicationAddress = additionalInfo.FirstPublicationAddress
		detailResp.AdditionalInfo.FirstPublicationAddressCode = additionalInfo.FirstPublicationAddressCode
		detailResp.AdditionalInfo.FirstPublicationDate = additionalInfo.FirstPublicationDate
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), detailResp)
	return
}

func DeleteCopyright(ctx *gin.Context) {
	var (
		delReq *digital_copyright.DeleteCopyrightReq
		err    error
	)
	if err = ctx.ShouldBind(&delReq); err != nil {
		zap.L().Error("DeleteCopyright err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if _, err = service.GrpcCopyrightImpl.DeleteCopyright(context.Background(), delReq); err != nil {
		zap.L().Error("CopyrightList err DeleteCopyright", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), nil)
	return
}

func CopyrightSchedule(ctx *gin.Context) {
	var (
		req  *digital_copyright.ScheduleReq
		resp *digital_copyright.ScheduleResp
		err  error
	)
	if err = ctx.ShouldBind(&req); err != nil {
		zap.L().Error("CopyrightSchedule err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if resp, err = service.GrpcCopyrightImpl.Schedule(context.Background(), req); err != nil {
		zap.L().Error("CopyrightList err DeleteCopyright", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp)
	return
}

// 版权状态信息
func CopyrightStatusInfo(ctx *gin.Context) {
	type StatusInfo struct {
		Status     string `json:"status"`
		PrecessNum uint8  `json:"precessNum"`
	}
	var data []StatusInfo
	var statusSlice []string = []string{
		"WAIT_SUBMIT",
		"COPYRIGHT_CHECK",
		"DIGITAL_CHECK",
		"DIGITAL_WAIT_PAY",
		"DIGITAL_REVIEW",
		"COMPLETED",
		"ALL_AMEND",
		"ALL_END"}
	for _, status := range statusSlice {
		var statusInfo1 StatusInfo
		statusInfo1.Status = status
		statusInfo1.PrecessNum = 1
		data = append(data, statusInfo1)
	}

	resp, err := service.GrpcCopyrightImpl.StatusInfo(context.Background(), &digital_copyright.StatusInfoReq{CopyrightId: ""})
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), data)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp.Data)
	return
}

func RefreshCopyright(ctx *gin.Context) {
	RefreshCopyrightV1()
}

func RefreshCopyrightV1() {
	var (
		listReq  *digital_copyright.SyncListReq
		listResp *digital_copyright.SyncListResp
		err      error
	)
	lockKey := "refreshCopyright"
	locked, _err := cache.RedisClient.SetNX(lockKey, time.Now().Unix(), time.Minute*30).Result()
	if _err != nil {
		zap.L().Error("RefreshCopyright err", zap.Error(_err))
		return
	}
	if !locked {
		return
	}
	defer func() {
		_, _ = cache.RedisClient.Del(lockKey).Result()
	}()
	var maxWorker = 20
	tasks := make(chan *digital_copyright.SyncListResp_Info, 100)
	for i := 0; i < maxWorker; i++ {
		wg.Add(1)
		go func() {
			defer wg.Done()
			for info := range tasks {
				processCopyrightInfo(info)
			}
		}()
	}
	for i := 1; i <= 100; i++ {
		listReq = &digital_copyright.SyncListReq{
			Page:     int32(i),
			PageSize: 100,
		}
		if listResp, err = service.GrpcCopyrightImpl.SyncList(context.Background(), listReq); err != nil {
			zap.L().Error("RefreshCopyright err CopyrightList", zap.Error(err))
			break
		}
		if listResp == nil || len(listResp.Data) < int(listReq.PageSize) {
			break
		}
		for _, info := range listResp.Data {
			tasks <- info
		}
	}
	close(tasks)
	wg.Wait()
}

func processCopyrightInfo(info *digital_copyright.SyncListResp_Info) {
	var logicCopyright = new(loigc_copyright.Copyright)
	var logicDigital = new(loigc_copyright.Digital)
	zap.L().Info("copyright list info", zap.Any("info", info))
	copyrightInfo := model_copyright.CopyrightInfo{
		CopyrightId:            info.CopyrightId,
		DciContentId:           info.DciContentId,
		DigitalRegisterId:      info.DigitalRegisterId,
		PreRegCertUrl:          info.PreRegCertUrl,
		ApplyFormUrl:           info.ApplyFormUrl,
		InvoiceUrlList:         info.InvoiceUrlList,
		DigitalRegisterCertUrl: info.DigitalRegisterCertUrl,
		DigitalWorkSampleUrl:   info.DigitalWorkSampleUrl,
	}
	switch info.Status {
	case "DEFINE_DCI_PRE_WAIT_SUBMIT":
	case "DCI_PRE_REG_FAIL":
		fallthrough
	case "DEFINE_DCI_PRE_SUBMIT", "DCI_PRE_REG_AUDITING", "DCI_PRE_REG_TOBE_CONFIRM": //提交dci
		_, _ = logicCopyright.PreregistrationInfo(info.DciContentId, info.CopyrightId)
	case "REVIEW", "DEFINE_DIGI_SUBMIT", "CHECK", "SUBMITED":
		idAll := pkgRequest.CopyrightIdAll{
			CopyrightId: info.CopyrightId,
		}
		_, _ = logicDigital.DciQueryRegistration(idAll, copyrightInfo)
	case "DCI_PRE_REG_FINISH":
		_, _ = logicDigital.DciRegistration(info.CopyrightId)
	case "PAY":
		idAll := pkgRequest.CopyrightIdAll{
			CopyrightId: info.CopyrightId,
		}
		if info.Amount == 0 {
			zap.L().Info("DciGetPayUrl 2")
			_, _ = logicDigital.DciGetPayUrl(idAll)
		}
		if info.PayStatus == "PAY_SUCCESS" {
			_, _ = logicDigital.DciQueryRegistration(idAll, copyrightInfo)
		} else {
			_, _ = logicDigital.DciGetPayQuery(idAll)
		}
	default:

	}
}

func CheckDciUser(ctx *gin.Context) {
	// 15262419073
	// 320981199411161742
	// 韩洁
	// national https://cdns.fontree.cn/fonchain-main/prod/image/2e126e4a-8920-4034-9904-00b2feaa879d/artist/90bca41a-2ae4-4fa3-9eb7-b4a3c2e9f351.jpg
	// cardface https://cdns.fontree.cn/fonchain-main/prod/image/2e126e4a-8920-4034-9904-00b2feaa879d/artist/94352ea8-f46f-4e85-8d72-9ed61b869873.jpg
	var logicDciUser = new(loigc_copyright.DciUser)
	var artistInfo *artist.ArtistInfoResponse = &artist.ArtistInfoResponse{}
	artistInfo.ProfileInfo = &artist.ProfileRequest{
		CardId: "320981199411161742",
		Phone:  "15262419073",
		Uid:    "define41-3249-1310-ee4d-3677861bc281",
		Name:   "韩洁",
	}
	artistInfo.MediaRequest = &artist.MediaRequest{
		CardFace:     "https://cdns.fontree.cn/fonchain-main/prod/image/2e126e4a-8920-4034-9904-00b2feaa879d/artist/94352ea8-f46f-4e85-8d72-9ed61b869873.jpg",
		CardNational: "https://cdns.fontree.cn/fonchain-main/prod/image/2e126e4a-8920-4034-9904-00b2feaa879d/artist/90bca41a-2ae4-4fa3-9eb7-b4a3c2e9f351.jpg",
	}
	resp, err := logicDciUser.CheckDciReg(artistInfo)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, "", resp)
	return
}

func CreatePregistration(ctx *gin.Context) {
	//dciUserId := "DUSANDBOX8240508000000555798" //韩洁测试
	dciUserId := "DU8240509000002198923" //韩洁正式
	authorName := "韩洁"
	mp3Url := "https://cdns.fontree.cn/fonchain-main/prod/image/2e126e4a-8920-4034-9904-00b2feaa879d/artist/183ef31d-a23a-44cf-bf93-6b305959c9a2.mp3"
	var logicDciUser = new(loigc_copyright.DciUser)
	wordFileId, err := logicDciUser.DciUploadUrl(mp3Url)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}

	req := &dci.CreateDciPreregistrationRequest{
		WorkName:     "时间的涟漪",
		DciUserId:    dciUserId,
		WorkCategory: "MUSIC",
		WorkFileId:   wordFileId,
		FileType:     "AUDIO",
		CreationInfo: &dci.DciCreationInfo{
			CreationNature:         "ORIGINAL",
			CreationCompletionDate: "2024-04-02",
			CreationCompletionCode: "320500",
		},
		PublicationInfo: &dci.DciPublicationInfo{
			PublicationStatus:    "PUBLISHED",
			FirstPublicationDate: "2024-05-10",
			FirstPublicationCode: "320500",
		},
		AuthorName:      authorName,
		AuthorSignature: authorName,
		//OriginalStatement:       "",
		RightInfo: &dci.DciRightInfo{
			RightScope:     "ALL",
			RightObtainWay: "ORIGINAL",
			OwnershipWay:   "PERSON",
		},
		PreRegistrationTrueWill: "PASSWORD",
		CopyrightOwnerIds:       []string{dciUserId},
	}
	zap.L().Info("CreateDciPreregistration", zap.Any("req", req))
	resp, err := service.GrpcDciImpl.CreateDciPreregistration(context.Background(), req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		zap.L().Error("dciArtworkPreregistration err CreateDciPreregistration", zap.Error(err))
		return
	}
	zap.L().Info("CreateDciPreregistration--", zap.Any("resp", resp))
	service.ResponseQuickMsg(ctx, e.Ok, "", resp)
	return
}

func UpdatePayInfoResult(ctx *gin.Context) {
	req := new(digital_copyright.UpdatePayInfoResultReq)
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcCopyrightImpl.UpdatePayInfoResult(context.Background(), req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		zap.L().Error("UpdatePayInfoResult err", zap.Error(err))
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, "", resp)
	return
}
