package exam

import (
	"errors"

	"github.com/fonchain_enterprise/fonchain-main/api/exam"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUtil"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	"github.com/gin-gonic/gin"
)

// 证书管理模块
var CertHandler = &certHandler{}

type certHandler struct {
}

// 创建Cert
func (a *certHandler) CreateCert(c *gin.Context) {
	var req exam.CertData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	if err := CetParamsValidate(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}
	loginInfoInterface, exist := c.Get("mLoginInfo")
	if !exist {
		service.Error(c, e.NotLogin, nil, "未登录")
		return
	}
	logininfo := loginInfoInterface.(model.LoginInfo)
	req.CreatorId = int64(logininfo.ID)
	req.CreatedBy = utils.IfGec(logininfo.NickName != "", logininfo.NickName, logininfo.Account)
	_, err := service.GrpcExamClientImpl.CreateCert(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcExamClientImpl.CreateCert Err", "创建失败")
		return
	}
	service.Success(c)
}

// 删除Cert
func (a *certHandler) DeleteCert(c *gin.Context) {
	var req exam.DeleteCertRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	loginInfoInterface, exist := c.Get("mLoginInfo")
	if !exist {
		service.Error(c, e.NotLogin, nil, "未登录")
		return
	}
	logininfo := loginInfoInterface.(model.LoginInfo)
	req.UpdaterId = int64(logininfo.ID)
	req.UpdatedBy = utils.IfGec(logininfo.NickName != "", logininfo.NickName, logininfo.Account)
	resp, err := service.GrpcExamClientImpl.DeleteCert(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "DeleteCert Err", utils.IfGec(resp.Msg == "", "更新失败", resp.Msg))
		return
	}
	service.Success(c)
}

// 更新Cert
func (a *certHandler) UpdateCert(c *gin.Context) {
	var req exam.CertData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	if err := CetParamsValidate(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}
	loginInfoInterface, exist := c.Get("mLoginInfo")
	if !exist {
		service.Error(c, e.NotLogin, nil, "未登录")
		return
	}
	logininfo := loginInfoInterface.(model.LoginInfo)
	req.UpdaterId = int64(logininfo.ID)
	req.UpdatedBy = utils.IfGec(logininfo.NickName != "", logininfo.NickName, logininfo.Account)
	resp, err := service.GrpcExamClientImpl.UpdateCert(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcExamClientImpl.UpdateCert Err", utils.IfGec(resp.Msg == "", "更新失败", resp.Msg))
		return
	}
	service.Success(c)
}

// 使用id查询Cert
func (a *certHandler) GetCertDetail(c *gin.Context) {
	var req exam.GetCertByIdRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	resp, err := service.GrpcExamClientImpl.GetCertDetail(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcExamClientImpl.GetCertDetail Err", "查询失败")
		return
	}
	service.Success(c, resp)
}

// 批量查询Cert
func (a *certHandler) GetCertList(c *gin.Context) {
	var req GetCertListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var protoReq = exam.GetCertListRequest{Query: &exam.CertData{}, EnableStatic: true}
	asUtil.RequestDataConvert(&req, &protoReq)
	resp, err := service.GrpcExamClientImpl.GetCertList(c, &protoReq)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetCertList Err", "查询失败")
		return
	}
	var data []GetCertListType
	for _, v := range resp.List {
		v := v
		data = append(data, GetCertListType{
			ID:        v.ID,
			CreatedAt: v.CreatedAt,
			UpdatedAt: v.UpdatedAt,
			DeletedAt: v.DeletedAt,
			Name:      v.Name,
			SampleUrl: v.SampleUrl,
			Thumbnail: v.Thumbnail,
			Desc:      v.Desc,
			ValidTime: v.ValidTime,
			CreatedBy: v.CreatedBy,
			CreatorId: v.CreatorId,
			UpdatedBy: v.UpdatedBy,
			UpdaterId: v.UpdaterId,
			OwnerNum:  v.StaticData.OwnerNum,
			Status:    v.StaticData.Status,
		})
	}
	service.ResponseList(c, data, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}
func CetParamsValidate(data *exam.CertData) error {
	if len([]rune(data.Name)) > 50 {
		return errors.New("名称最多50个字符")
	}
	if len([]rune(data.Desc)) > 200 {
		return errors.New("说明最多200个字符")
	}
	if data.SampleUrl == "" {
		return errors.New("请上传证书样本")
	}
	if data.Thumbnail == "" {
		return errors.New("请上传证书缩略图")
	}
	return nil
}

// 查看我得证书
func (a *certHandler) GetMyCert(c *gin.Context) {
	var req exam.GetMyCertRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.UserID = int64(userInfo.ID)

	resp, err := service.GrpcExamClientImpl.GetMyCert(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcExamClientImpl.GetMyCert Err", "查询失败")
		return
	}
	service.Success(c, resp)
}

// 使用id查询Cert
func (a *certHandler) CheckCert(c *gin.Context) {
	var req exam.ExamTaskCheckPageRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	resp, err := service.GrpcExamClientImpl.CheckCert(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcExamClientImpl.CheckCert Err", "查询失败")
		return
	}
	service.Success(c, resp)
}
